package org.mevenide.idea.util.psi;

import com.intellij.psi.PsiTreeChangeEvent;

/**
 * @author Arik
 */
public interface SimplePsiListener {

    /**
     * Returns the source of the current modification taking place.
     *
     * <p>This method is called during PSI event processing, or during a Swing event processing to
     * determine if the PSI/Swing event was generated by the user directly or by the listeners.</p>
     *
     * <p>For example, if the user modifies the POM via the Swing widgets, than the swing widgets
     * modify the PSI tree, which generate a PSI event. In the PSI event we must know that the event
     * was originally generated by the source editor and therefor not trigger the Swing models (or
     * an infinite loop will occur).</p>
     *
     * @return the modification source
     */
    ModificationSource getModificationSource();

    /**
     * Sets the source of the current event.
     *
     * @param pSource the event source
     */
    void setModificationSource(ModificationSource pSource);

    /**
     * This method must synchronize this model with the PSI model, where the PSI model acts as the
     * source and this model as the destination.
     *
     * <p>A possible approach is to completely build this model's values from the PSI tree, or the
     * incremently apply changes from the supplied PSI event object.</p>
     *
     * <p>If the {@code pEvent} parameter is {@code null}, then a complete rebuild of the model from
     * the PSI tree should be performed.* </p>
     *
     * @param pType the PSI event type
     * @param pEvent     the PSI event object. This <i>may</i> be {@code null}
     */
    void refreshModel(PsiEventType pType, PsiTreeChangeEvent pEvent);
}
